package com.ahazou.photo_editor_sdk

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import ly.img.android.pesdk.PhotoEditorSettingsList
import ly.img.android.pesdk.assets.font.basic.FontPackBasic
import android.net.Uri
import android.os.Environment
import android.util.Log
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry
import ly.img.android.pesdk.backend.decoder.ImageSource
import ly.img.android.pesdk.backend.model.config.ImageStickerAsset
import ly.img.android.pesdk.backend.model.constant.Directory.DCIM
import ly.img.android.pesdk.backend.model.state.AssetConfig
import ly.img.android.pesdk.backend.model.state.LoadSettings
import ly.img.android.pesdk.backend.model.state.SaveSettings
import ly.img.android.pesdk.backend.model.state.manager.SettingsList
import ly.img.android.pesdk.ui.activity.ImgLyIntent
import ly.img.android.pesdk.ui.activity.PhotoEditorBuilder
import ly.img.android.pesdk.ui.model.state.*
import ly.img.android.pesdk.ui.panels.item.ImageStickerItem
import ly.img.android.pesdk.ui.panels.item.StickerCategoryItem
import ly.img.android.pesdk.ui.panels.item.ToolItem
import ly.img.android.serializer._3._0._0.PESDKFileWriter
import java.io.File
import java.io.IOException
import java.lang.ref.WeakReference
import java.util.ArrayList

/** PhotoEditorSdkPlugin */
class PhotoEditorSdkPlugin:
        ActivityAware,
        FlutterPlugin,
        MethodCallHandler,
        PluginRegistry.ActivityResultListener {

  private val arrLogos = ArrayList<CustomImageStickerAsset>()
  private val arrStickers = ArrayList<CustomImageStickerAsset>()

  companion object {
    lateinit var activity: WeakReference<Activity>
    lateinit var resultG: Result
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.binaryMessenger, "photo_editor_sdk")
    channel.setMethodCallHandler(PhotoEditorSdkPlugin())
  }

  override fun onDetachedFromActivity() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = WeakReference(binding.activity)
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    resultG = result
    when ( call.method ) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "addAllContents" -> {
        val args = call.arguments as Map<*, *>
        val logos: ArrayList<*> = args["logos"] as ArrayList<*>
        val stickers: ArrayList<*> = args["stickers"] as ArrayList<*>
        retrieveAssets(logos, arrLogos)
        retrieveAssets(stickers, arrStickers)
      }

      "editImage" -> {
        val args = call.arguments as? String
        val uri = Uri.parse(args)
        openEditor(uri)
      }
      else -> result.notImplemented()
    }
  }

  private fun retrieveAssets(list: ArrayList<*>, retrieved: ArrayList<CustomImageStickerAsset>) {
    list.forEach {
      Glide
              .with(activity.get()!!)
              .asFile()
              .load(it)
              .listener(object : RequestListener<File> {
                override fun onLoadFailed(e: GlideException?, model: Any?,
                                          target: Target<File>?, isFirstResource: Boolean): Boolean {
                  return false
                }

                override fun onResourceReady(resource: File?, model: Any?, target: Target<File>?,
                                             dataSource: DataSource?, isFirstResource: Boolean): Boolean {
                  val uri = Uri.fromFile(resource)
                  val imageStickerConfig =
                          ImageStickerAsset.createTemporaryStickerAsset(uri)
                  retrieved.add(CustomImageStickerAsset(imageStickerConfig, uri))
                  return false
                }
              })
              .submit()
    }
  }

  private fun createPesdkSettingsList(inputImage: Uri?) =
          PhotoEditorSettingsList()
                  .configure<UiConfigText> {
                    it.setFontList(FontPackBasic.getFontPack())
                  }
                  .configure<SaveSettings> {
                    // Set custom editor image export settings
                    it.setExportDir(DCIM, "AhazouPro")
                    it.setExportPrefix("result_")
                    it.savePolicy = SaveSettings.SavePolicy.RETURN_ALWAYS_ONLY_OUTPUT
                  }
                  .configure<LoadSettings> {
                    it.source = inputImage
                    it.isDeleteProtectedSource = true
                  }
                  .configure<UiConfigTheme> {
                    it.theme =  R.style.Imgly_Theme_TopActionBar_NoFullscreen
                  }
                  .configure<UiConfigMainMenu> {
                    it.setToolList(
                            ToolItem("imgly_tool_sticker_selection",
                                    R.string.pesdk_sticker_title_name, ImageSource.create(R.drawable.imgly_icon_tool_sticker)),
                            ToolItem("imgly_tool_text",
                                    R.string.pesdk_text_title_name, ImageSource.create(R.drawable.imgly_icon_tool_text))
                    )
                  }
                  .configure<UiConfigSticker> {
                    val arrLogoSticker = ArrayList<StickerCategoryItem>()
                    if( arrLogos.isNotEmpty() ) {
                      arrLogoSticker.add(StickerCategoryItem(
                              "logos_id",
                              "logos",
                              ImageSource.create(arrLogos[0].uri),
                              arrLogos.map { s -> ImageStickerItem.createFromAsset(s.image) }
                      ))
                    }

                    if( arrStickers.isNotEmpty() ) {
                      arrLogoSticker.add(StickerCategoryItem(
                              "stickers_id",
                              "stickers",
                              ImageSource.create(arrStickers[0].uri),
                              arrStickers.map { s -> ImageStickerItem.createFromAsset(s.image) }
                      ))
                    }
                    it.setStickerLists(arrLogoSticker)
                  }
                  .configure<AssetConfig> {
                    arrLogos.map { logo -> it.addAsset(logo.image) }
                    arrStickers.map { sticker -> it.addAsset(sticker.image) }
                  }

  val PESDK_RESULT = 1000

  private fun openEditor(inputImage: Uri?) {
    val settingsList = createPesdkSettingsList(inputImage)

    PhotoEditorBuilder(activity.get())
            .setSettingsList(settingsList)
            .startActivityForResult(activity.get(), PESDK_RESULT)
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {

    if (resultCode == Activity.RESULT_OK && requestCode == PESDK_RESULT) { // Editor has saved an Image.

      val resultURI = data.getParcelableExtra<Uri?>(ImgLyIntent.RESULT_IMAGE_URI)?.also {
        // Scan result uri to show it up in the Gallery
        // activity.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE).setData(it))
      }

      val sourceURI = data.getParcelableExtra<Uri?>(ImgLyIntent.SOURCE_IMAGE_URI)?.also {
        // Scan source uri to show it up in the Gallery
        // sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE).setData(it))
      }

      Log.i("PESDK", "Source image is located here $sourceURI")
      Log.i("PESDK", "Result image is located here $resultURI")

      // TODO: Do something with the result image

      // OPTIONAL: read the latest state to save it as a serialisation
      val lastState = data.getParcelableExtra<SettingsList>(ImgLyIntent.SETTINGS_LIST)
      try {
        PESDKFileWriter(lastState).writeJson(File(
                Environment.getExternalStorageDirectory(),
                "serialisationReadyToReadWithPESDKFileReader.json"
        ))
      } catch (e: IOException) {
        e.printStackTrace()
      }

    } else if (resultCode == Activity.RESULT_CANCELED && requestCode == PESDK_RESULT) {

      // Editor was canceled
      val sourceURI = data.getParcelableExtra<Uri?>(ImgLyIntent.SOURCE_IMAGE_URI)
      // TODO: Do something...
    }

    return true
  }

  data class CustomImageStickerAsset(var image: ImageStickerAsset, val uri: Uri)
}